@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2020 - pancake */
 
 #ifndef R2_CORE_H
 #define R2_CORE_H
@@ -30,13 +30,17 @@
 #include "r_util/r_print.h"
 #include "r_crypto.h"
 #include "r_bind.h"
+#include "r_util/r_annotated_code.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 R_LIB_VERSION_HEADER(r_core);
 
+#define R_CORE_CMD_OK 0
+#define R_CORE_CMD_INVALID -1
 #define R_CORE_CMD_EXIT -2
+
 #define R_CORE_BLOCKSIZE 0x100
 #define R_CORE_BLOCKSIZE_MAX 0x3200000 /* 32MB */
 
@@ -79,10 +83,6 @@ R_LIB_VERSION_HEADER(r_core);
 #define RTR_PROTOCOL_HTTP 3
 #define RTR_PROTOCOL_UNIX 4
 
-#define RTR_RAP_OPEN   0x01
-#define RTR_RAP_CMD    0x07
-#define RTR_RAP_REPLY  0x80
-
 #define RTR_MAX_HOSTS 255
 
 /* visual mode */
@@ -263,7 +263,7 @@ typedef struct r_core_t {
 	RCmdDescriptor root_cmd_descriptor;
 	RList/*<RCmdDescriptor>*/ *cmd_descriptors;
 	RAnal *anal;
-	RAsm *assembler;
+	RAsm *rasm;
 	/* ^^ */
 	RCoreTimes *times;
 	RParse *parser;
@@ -333,13 +333,13 @@ typedef struct r_core_t {
 
 	RMainCallback r_main_radare2;
 	// int (*r_main_radare2)(int argc, char **argv);
-	int (*r_main_rafind2)(int argc, char **argv);
-	int (*r_main_radiff2)(int argc, char **argv);
-	int (*r_main_rabin2)(int argc, char **argv);
-	int (*r_main_rarun2)(int argc, char **argv);
-	int (*r_main_ragg2)(int argc, char **argv);
-	int (*r_main_rasm2)(int argc, char **argv);
-	int (*r_main_rax2)(int argc, char **argv);
+	int (*r_main_rafind2)(int argc, const char **argv);
+	int (*r_main_radiff2)(int argc, const char **argv);
+	int (*r_main_rabin2)(int argc, const char **argv);
+	int (*r_main_rarun2)(int argc, const char **argv);
+	int (*r_main_ragg2)(int argc, const char **argv);
+	int (*r_main_rasm2)(int argc, const char **argv);
+	int (*r_main_rax2)(int argc, const char **argv);
 } RCore;
 
 // maybe move into RAnal
@@ -398,10 +398,9 @@ R_API int r_core_lines_initcache (RCore *core, ut64 start_addr, ut64 end_addr);
 R_API int r_core_lines_currline (RCore *core);
 R_API void r_core_prompt_loop(RCore *core);
 R_API ut64 r_core_pava(RCore *core, ut64 addr);
-R_API void run_pending_anal(RCore *core);
 R_API int r_core_cmd(RCore *core, const char *cmd, int log);
 R_API int r_core_cmd_task_sync(RCore *core, const char *cmd, bool log);
-R_API char *r_core_editor (const RCore *core, const char *file, const char *str);
+R_API char *r_core_editor(const RCore *core, const char *file, const char *str);
 R_API int r_core_fgets(char *buf, int len);
 R_API RFlagItem *r_core_flag_get_by_spaces(RFlag *f, ut64 off);
 R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
@@ -442,6 +441,7 @@ R_API bool r_core_prevop_addr(RCore* core, ut64 start_addr, int numinstrs, ut64*
 R_API ut64 r_core_prevop_addr_force(RCore *core, ut64 start_addr, int numinstrs);
 R_API bool r_core_visual_hudstuff(RCore *core);
 R_API int r_core_visual_classes(RCore *core);
+R_API int r_core_visual_anal_classes(RCore *core);
 R_API int r_core_visual_types(RCore *core);
 R_API int r_core_visual(RCore *core, const char *input);
 R_API int r_core_visual_graph(RCore *core, RAGraph *g, RAnalFunction *_fcn, int is_interactive);
@@ -465,8 +465,9 @@ R_API void r_core_set_asmqjmps(RCore *core, char *str, size_t len, int i);
 R_API char* r_core_add_asmqjmp(RCore *core, ut64 addr);
 
 R_API void r_core_anal_type_init(RCore *core);
+R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
 R_API void r_core_anal_inflags (RCore *core, const char *glob);
-R_API int cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
+R_API bool cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
 R_API void r_core_anal_cc_init(RCore *core);
 R_API void r_core_anal_paths(RCore *core, ut64 from, ut64 to, bool followCalls, int followDepth, bool is_json);
 R_API void r_core_anal_esil_graph(RCore *core, const char *expr);
@@ -506,7 +507,7 @@ R_API int r_core_file_list(RCore *core, int mode);
 R_API int r_core_file_binlist(RCore *core);
 R_API bool r_core_file_bin_raise(RCore *core, ut32 num);
 R_API int r_core_seek_delta(RCore *core, st64 addr);
-R_API int r_core_extend_at(RCore *core, ut64 addr, int size);
+R_API bool r_core_extend_at(RCore *core, ut64 addr, int size);
 R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size);
 R_API int r_core_write_op(RCore *core, const char *arg, char op);
 R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op);
@@ -558,7 +559,7 @@ R_API char *r_core_cmd_str(RCore *core, const char *cmd);
 R_API int r_core_cmd_foreach(RCore *core, const char *cmd, char *each);
 R_API int r_core_cmd_foreach3(RCore *core, const char *cmd, char *each);
 R_API char *r_core_op_str(RCore *core, ut64 addr);
-R_API RAnalOp *r_core_op_anal(RCore *core, ut64 addr);
+R_API RAnalOp *r_core_op_anal(RCore *core, ut64 addr, RAnalOpMask mask);
 R_API char *r_core_disassemble_instr(RCore *core, ut64 addr, int l);
 R_API char *r_core_disassemble_bytes(RCore *core, ut64 addr, int b);
 
@@ -566,12 +567,8 @@ R_API char *r_core_disassemble_bytes(RCore *core, ut64 addr, int b);
 R_API RList *r_core_get_func_args(RCore *core, const char *func_name);
 R_API void r_core_print_func_args(RCore *core);
 R_API char *resolve_fcn_name(RAnal *anal, const char * func_name);
-
 R_API int r_core_get_stacksz(RCore *core, ut64 from, ut64 to);
 
-/* cmd_type.c */
-R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
-
 /* anal.c */
 R_API RAnalOp* r_core_anal_op(RCore *core, ut64 addr, int mask);
 R_API void r_core_anal_esil(RCore *core, const char *str, const char *addr);
@@ -617,7 +614,7 @@ R_API RList *r_core_anal_fcn_get_calls (RCore *core, RAnalFunction *fcn); // get
 
 /*tp.c*/
 R_API void r_core_anal_type_match(RCore *core, RAnalFunction *fcn);
-R_API RStrBuf *var_get_constraint (RAnal *a, RAnalVar *var);
+R_API RStrBuf *var_get_constraint(RAnal *a, RAnalFunction *fcn, RAnalVar *var);
 
 /* asm.c */
 #define R_MIDFLAGS_SHOW 1
@@ -679,7 +676,7 @@ R_API bool r_core_project_open(RCore *core, const char *file, bool thready);
 R_API int r_core_project_cat(RCore *core, const char *name);
 R_API int r_core_project_delete(RCore *core, const char *prjfile);
 R_API int r_core_project_list(RCore *core, int mode);
-R_API bool r_core_project_save_rdb(RCore *core, const char *file, int opts);
+R_API bool r_core_project_save_script(RCore *core, const char *file, int opts);
 R_API bool r_core_project_save(RCore *core, const char *file);
 R_API char *r_core_project_info(RCore *core, const char *file);
 R_API char *r_core_project_notes_file (RCore *core, const char *file);
@@ -922,12 +919,44 @@ R_API void r_core_autocomplete_free(RCoreAutocomplete *obj);
 R_API void r_core_autocomplete_reload (RCore *core);
 R_API RCoreAutocomplete *r_core_autocomplete_find(RCoreAutocomplete *parent, const char* cmd, bool exact);
 R_API bool r_core_autocomplete_remove(RCoreAutocomplete *parent, const char* cmd);
-R_API void r_core_anal_propagate_noreturn(RCore *core);
+R_API void r_core_anal_propagate_noreturn(RCore *core, ut64 addr);
 
 /* PLUGINS */
 extern RCorePlugin r_core_plugin_java;
 extern RCorePlugin r_core_plugin_a2f;
 
+/* DECOMPILER PRINTING FUNCTIONS */
+/**
+ * @brief Prints the data contained in the specified RAnnotatedCode in JSON format.
+ * 
+ * The function will print the output in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_core_annotated_code_print_json(RAnnotatedCode *code);
+/**
+ * @brief Prints the decompiled code from the specified RAnnotatedCode.
+ * 
+ * This function is used for printing the output of commands pdg and pdgo.
+ * It can print the decompiled code with or without offsets. If line_offsets is a null pointer,
+ * the output will be printed without offsets (pdg), otherwise, the output will be
+ * printed with offsets.
+ * This function will print the output in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ * @param line_offsets Pointer to a @ref RVector that contains offsets for the decompiled code.
+ */
+R_API void r_core_annotated_code_print(RAnnotatedCode *code, RVector *line_offsets);
+/**
+ * @brief  Prints the decompiled code as comments
+ * 
+ * This function is used for the output of command pdg*
+ * Output will be printed in console using the function r_cons_printf();
+ * 
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_core_annotated_code_print_comment_cmds(RAnnotatedCode *code);
+
 #endif
 
 #ifdef __cplusplus
